@@ -4,7 +4,7 @@ use strict;
 use Carp;
 use base qw(Unicode::Collate);
 
-our $VERSION = '0.75';
+our $VERSION = '0.79';
 
 use File::Spec;
 
@@ -12,15 +12,14 @@ use File::Spec;
 my $PL_EXT  = '.pl';
 
 my %LocaleFile = map { ($_, $_) } qw(
-   af ar az ca cs cy da eo es et fi fil fo fr ha haw
-   hr hu hy ig is ja kk kl ko lt lv mt nb nn nso om pl ro ru
-   se sk sl sq sv tn to tr uk vi wo yo zh
+   af ar as az be bg ca cs cy da eo es et fi fil fo fr gu
+   ha haw hr hu hy ig is ja kk kl ko lt lv mt nb nn nso om
+   pl ro ru se sk sl sq sv tn to tr uk vi wo yo zh
 );
    $LocaleFile{'default'}         = '';
    $LocaleFile{'de__phonebook'}   = 'de_phone';
    $LocaleFile{'es__traditional'} = 'es_trad';
-   $LocaleFile{'be'} = 'ru';
-   $LocaleFile{'bg'} = 'ru';
+   $LocaleFile{'fi__phonebook'}   = 'fi_phone';
    $LocaleFile{'mk'} = 'ru';
    $LocaleFile{'sr'} = 'ru';
    $LocaleFile{'zh__big5han'}   = 'zh_big5';
@@ -84,6 +83,73 @@ sub new {
 1;
 __END__
 
+MEMORANDA for developing
+
+locale		based CLDR
+----------------------------------------------------------------------------
+af		2.0 = 1.8.1
+ar		2.0
+as		2.0 = 1.8.1
+az		2.0 = 1.8.1
+be		2.0
+bg		2.0
+ca		2.0 = 1.8.1 (alt="proposed" type="standard")
+cs		2.0 = 1.8.1 (type="standard")
+cy		2.0 = 1.8.1
+da		2.0 = 1.8.1 (type="standard") [AA's to pass CLDR tests]
+de__phonebook	2.0 (type="phonebook")
+eo		2.0 = 1.8.1
+es		2.0 (type="standard")
+es__traditional 2.0 = 1.8.1 (type="traditional")
+et		2.0 = 1.8.1
+fi		2.0 = 1.8.1 (type="standard" alt="proposed")
+fi__phonebook	2.0 = 1.8.1 (type="phonebook")
+fil		2.0 (type="standard") = 1.8.1
+fo		2.0 = 1.8.1 (alt="proposed" type="standard")
+fr		2.0 (fr_CA, backwards="on")
+gu		2.0 (type="standard")
+ha
+haw
+hr
+hu
+hy
+ig
+is
+ja
+kk
+kl
+ko
+lt
+lv
+mk
+mt
+nb
+nn
+nso
+om
+pl
+ro
+ru
+se
+sk
+sl
+sq
+sr
+sv
+tn
+to
+tr
+uk
+vi
+wo
+yo
+zh
+zh__big5han
+zh__gb2312han
+zh__pinyin
+zh__stroke
+----------------------------------------------------------------------------
+
 =head1 NAME
 
 Unicode::Collate::Locale - Linguistic tailoring for DUCET via Unicode::Collate
@@ -190,6 +256,7 @@ this method returns a string C<'default'> meaning no special tailoring.
     ----------------------------------------------------------
       af                Afrikaans
       ar                Arabic
+      as                Assamese
       az                Azerbaijani (Azeri)
       be                Belarusian
       bg                Bulgarian
@@ -202,10 +269,12 @@ this method returns a string C<'default'> meaning no special tailoring.
       es                Spanish
       es__traditional   Spanish ('ch' and 'll' as a grapheme)
       et                Estonian
-      fi                Finnish
+      fi                Finnish (v and w are primary equal)
+      fi__phonebook     Finnish (v and w as separate characters)
       fil               Filipino
       fo                Faroese
       fr                French
+      gu                Gujarati
       ha                Hausa
       haw               Hawaiian
       hr                Croatian
